import {
  FisAppRequestMessage,
  ProducerType,
  Subscription
} from '../../fisjs/src/fisappmessagejs/fis/fismessagetype';
import {
  AppMessageCreator as AppMessage
} from '../../fisjs/src/fisappmessagejs/message/appmessagecreator';
import {
  AppMessageType, Command,
  CommandMessageParameter
} from "../../fisjs/src/fisappmessagejs/message/request/apprequestmessagetype";


const basic = (ucpId: string | null) => ({
  security: {
    ucpId: ucpId ? ucpId : '---'
  },
  producer: {
    type: ProducerType.UI,
    origin: {
      userApplication: {
        userAppId: "ui-123",
        userAppName: "Client"
      }
    }
  }
});


/** create new message */
export function getMessage(param: any, ucpId: string | null = null): FisAppRequestMessage {
  return AppMessage.create({
    ...basic(ucpId),
    ...param
  } as CommandMessageParameter) as FisAppRequestMessage;  
}

export function getLoginMessageByGoogleIdToken(idToken: string, timeout: number|null = null){
  const basic: any = {
    messageType: AppMessageType.Command, 
    messageName: "Login", 
    command: Command.Start,
    data: {
      authenticationType: 'google_id_token',
      idToken
    },
  }
  if(timeout != null) basic.data['timeout'] = timeout

  return getMessage(basic);
}

export function getLoginMessage(timeout: number|null = null){
  const basic: any = {
    messageType: AppMessageType.Command, 
    messageName: "Login", 
    command: Command.Start,
    data: {
      authenticationType: 'google',
      email: 'legit@gmail.com'
    },
  }
  if(timeout != null) basic.data['timeout'] = timeout

  return getMessage(basic);
}

export function getLogoutMessage(ucpId: string) {
  return getMessage({
    messageType: AppMessageType.Command,
    messageName: "Logout",
    command: Command.Logout,
    data: { },
  }, ucpId);
}

export function getSubPropertyChangeMessage(ucpId: string) {
  return getMessage({
    messageType: AppMessageType.Subscription,
    messageName: "subscribe_property_change",
    subscription: Subscription.General,
    data: {
      action: "subscribe",
      query: {
        FisCommands: {
          __args: {
            where: { category: 'property_change' },
          }
        },
      }
    },
  }, ucpId);
}

export function getNewUserProfile(ucpId: string) {
  return getMessage({
    "messageType": "Command",
    "messageName": "New User Profile (cmd)",
    "command": "New",
    "data": {
        "serviceId": "User Profile"
    }
  }, ucpId);
}

export function getNewPurchaseRequisition(ucpId: string) {
  return getMessage({
      "messageType": "Command",
      "messageName": "New Purchase Requisition (cmd)",
      "command": "New",
      "data": {
          "serviceId": "Purchase Requisition"
      }
  }, ucpId)
}

export function getSetPurchaseRequisitionDocDt(ucpId: string, value: string = "2021-01-10Z") {
  return getMessage({
      "messageType": "Command",
      "messageName": "Change document date to 2021 Jan 10.",
      "command": "SetItem",
      "data": {
          "serviceId": "Purchase Requisition",
          "alias": "header",
          "notifyChanged": true,
          "column": {
            "row": 1,
            "name": "doc_dt",
            "value": value,
          }
      }
  }, ucpId)
}

export function getSetUserProfileUserName(ucpId: string, value: string = "abcxyyz") {
  return getMessage({
      "messageType": "Command",
      "messageName": "Set User name (cmd)",
      "command": "SetItem",
      "data": {
          "serviceId": "User Profile",
          "alias": "header",
          "notifyChanged": true,
          "column": {
            "row": 1,
            "name": "user_name",
            "value": value,
          }
      }
  }, ucpId)
}

export function getCancelUserProfile(ucpId: string ) {
  return getMessage({
      "messageType": "Command",
      "messageName": "Cancel User Profile.",
      "command": "CancelChanges",
      "data": {
          "serviceId": "User Profile",
      }
  }, ucpId)
}

export function getCancelPurchaseRequisition(ucpId: string ) {
  return getMessage({
      "messageType": "Command",
      "messageName": "Cancel Purchase Requisiton.",
      "command": "CancelChanges",
      "data": {
          "serviceId": "Purchase Requisition",
      }
  }, ucpId)
}