import { Subject } from "rxjs";
import * as io from 'socket.io-client';

/* eslint-disable */
export class TestSocketIoClient {
  socket = null
  responses

  disconnect() {
    this.socket.close();
    this.socket.disconnect();
  }

  async connect(url, options = {} ) {
    const self = this;
    return new Promise((resolve, reject) =>{
      this.responses = new Subject()
  
      const ws = io.connect(url, options)
            
      // ws.on('connecting',     (x) => { console.info('connecting', x);     });
      // ws.on('connect_failed', (x) => { console.info('connect_failed',x);  });
      // ws.on('error',          (x) => { console.info('error', x);          });
      // ws.on('disconnect',     (x) => { console.info('disconnect',x);      });

      // https://socket.io/docs/v2/client-api/#Event-%E2%80%98connect-error%E2%80%99
      
      // ws.on('pong',              (x) => { console.info('pong',x);                });
      // ws.on('ping',              (x) => { console.info('ping',x);                });
      // ws.on('reconnect_failed',  (x) => { console.info('reconnect_failed', x);   });
      // ws.on('reconnecting',      (x) => { console.info('reconnecting', x);       });
      // ws.on('reconnect_attempt', (x) => { console.info('reconnect_attempt', x);  });
      // ws.on('reconnect',         (x) => { console.info('reconnect', x);          });
      // ws.on('connect_timeout',   (x) => { console.info('connect_timeout',x);     });
      ws.on('connect_error', (x) => {
        console.info('connect_error',x);
        reject(x)
      });

      ws.on('connect', (x) => {
        console.info('connect', ws.id)
        this.socket = ws;
        resolve(ws.id)
      });
      
      ws.on('response', (data) => { this.responses.next(data)  });
      
    })
  }

  
  getStreamOfResponsesFromRequest(request) {
    const subject = new Subject();
    
    const id = this.sendRequest(request);
    const subscription = this.responses.subscribe({
      next: (data) => {
        if ( data?.id != id ) { return }
        if ( data?.complete ) return resultsComplete();
        
        const response = JSON.parse( data?.message ? data.message : {} )
        subject.next(response)
      },
      complete: () => resultsComplete(),
      error: () => resultsComplete(),
    })

    function resultsComplete(){
      subject.complete();
      try { subscription.unsubscribe() } 
      catch (error) { console.info('error', error) }
    }

    return subject;
  }

  getAllResponsesFromRequest(request) {
    const id = this.sendRequest(request);
    const results = []

    return new Promise( (resolve) => {
      const subscription = this.responses.subscribe({
        next: (data) => {
          console.info('got next', data)
          if ( data?.id != id ) { return }
          if ( data.complete ) return resultsComplete()

          const response = JSON.parse( data?.message ? data.message : {} )
          results.push(response);
        },
        complete: () => resultsComplete(),
        error: () => resultsComplete(),
      })

      function resultsComplete(){
        console.info('got result', results)
        resolve(results)
        try { subscription.unsubscribe() }
        catch (error) { console.info('error', error) }
      }
    });

  }

  sendRequest(request) {
    const id = Math.round(Math.random() * 100000);
    setTimeout(()=> {
      this.socket.emit('request', {
        id,
        message: JSON.stringify(request)
      })
    }, 0);
    return id;
  }
}