/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    Message, AppMessageType, AppMessageHeaderOptions,
    FisCommandMessageParameter as MessageParameter, FisCommandMessage
} from './fisrequestmessagetype';
import { AppMessageKind } from '../message/common/appmessagekind';

/**
 * App Fis command message.
 *
 * @class AppFisCommandMessage
 */
export class AppFisCommandMessage extends AppMessageKind {
    /**
     * Create new app Fis command message.
     *
     * @class AppFisCommandMessage
     * @method constructor
     * @param options {AppMessageHeaderOptions} - Message options. 
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        this.messageType = AppMessageType.FisCommand;
    }

    /**
     * App Fis command Message.
     * 
     * @class AppFisCommandMessage
     * @property commandMessage
     * @type {FisCommandMessage} - App Fis command Message.
     */
    protected commandMessage: FisCommandMessage;

    /**
     * Create new Fis command message.
     *
     * @class AppFisCommandMessage
     * @method createMessage
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisCommandMessage} - New Fis command message.
     */
    protected createMessage(messageParameter: MessageParameter): FisCommandMessage {
        try {
            this.commandMessage = {} as FisCommandMessage;
            return this.commandMessage;
        }
        catch (e) {
            throw e;
        }
    }

}

/**
 * Create new App Fis command message.
 *
 * @function createFisCommandMessage
 * @param messageParameter {MessageParameter} - Message parameters. 
 * @return {Message} - App Fis command message.
 */
export function createFisCommandMessage(messageParameter: MessageParameter): Message {
    try {
        return new AppFisCommandMessage().create(messageParameter);
    }
    catch (e) {
        throw e;
    }
}
