/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    AppMessageType, GenericHeader, GenericHeaderOf, AppMessageHeaderOptions,
    FisCommandMessageParameter as MessageParameter, FisCommandMessageHeader
} from './fisrequestmessagetype';
import { AppMessageHeaderKind } from '../message/common/appmessageheaderkind';

/**
 * Fis command message header.
 *
 * @class AppFisCommandMessageHeader
 */
export class AppFisCommandMessageHeader extends AppMessageHeaderKind {
    /**
     * Create new Fis command message header.
     *
     * @class AppFisCommandMessageHeader
     * @param options {AppMessageHeaderOptions} - Message header options. 
     * @method constructor
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        // App message base header type is Fis request header.
        this.messageBaseHeaderType = AppMessageType.FisRequest;
        this.messageType = AppMessageType.FisCommand;
        this.permissibleMessageType = AppMessageType;
        if (!this.options) { this.options = {}; }
        this.options.permissibleMessageType = this.permissibleMessageType;
    }

    /**
     * Create new Fis command header.
     *
     * @class AppFisCommandMessageHeader
     * @method createHeader
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisCommandMessageHeader} - New command header.
     */
    protected createHeader(messageParameter: MessageParameter): GenericHeader {
        try {
            let header: FisCommandMessageHeader = {} as FisCommandMessageHeader;
            header.command = (messageParameter && messageParameter.command);
            return header as GenericHeaderOf<FisCommandMessageHeader>;
        }
        catch (e) {
            throw e;
        }
    }
}
