/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import { FisMessageHeader } from './fismessagetype';
import { FisMessageParameter as MessageParameter } from './fismessageparameter';

/**
 * App Fis message header base.
 *
 * @class AppFisMessageHeaderBase
 */
export class AppFisMessageHeaderBase {
    /**
     * App Fis message header base.
     * Information required for message integration with FIS back office. 
     * 
     * @class AppFisMessageHeaderBase
     * @property fisMessageHeader
     * @type {FisMessageHeader}
     */
    protected fisMessageHeader: FisMessageHeader;

    /**
     * Create new App Fis message header base.
     *
     * @class AppFisMessageHeaderBase
     * @method create
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisMessageHeader} - App Fis message header base.
     */
    public create(messageParameter: MessageParameter): FisMessageHeader {
        try {
            this.fisMessageHeader = {} as FisMessageHeader;
            this.fisMessageHeader.instanceId = (messageParameter && messageParameter.instanceId) || null;
            this.fisMessageHeader.serviceId =  (messageParameter && messageParameter.serviceId) || null;
            this.fisMessageHeader.userId =  (messageParameter && messageParameter.userId) || null;;
            this.fisMessageHeader.dataSourceTiming =  (messageParameter && messageParameter.dataSourceTiming) || "";
            return this.fisMessageHeader;
        }
        catch (e) {
            throw e;
        }
    }
}

/**
 * Create new App Fis message header base.
 *
 * @function createFisMessageHeader
 * @param messageParameter {MessageParameter} - Message parameters. 
 * @return {FisMessageHeader} - App Fis message header base.
 */
export function createFisMessageHeader(messageParameter: MessageParameter): FisMessageHeader {
    try {
        return new AppFisMessageHeaderBase().create(messageParameter);
    }
    catch (e) {
        throw e;
    }
};
