/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    Message, AppMessageType, AppMessageHeaderOptions,
    FisQueryMessageParameter as MessageParameter, FisQueryMessage
} from './fisrequestmessagetype';
import { AppMessageKind } from '../message/common/appmessagekind';

/**
 * App Fis query message.
 *
 * @class AppFisQueryMessage
 */
export class AppFisQueryMessage extends AppMessageKind {
    /**
     * Create new app Fis query message.
     *
     * @class AppFisQueryMessage
     * @method constructor
     * @param options {AppMessageHeaderOptions} - Message options. 
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        this.messageType = AppMessageType.FisQuery;
    }

    /**
     * App Fis query Message.
     * 
     * @class AppFisQueryMessage
     * @property queryMessage
     * @type {FisQueryMessage} - App Fis query Message.
     */
    protected queryMessage: FisQueryMessage;

    /**
     * Create new Fis query message.
     *
     * @class AppFisQueryMessage
     * @method createMessage
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisQueryMessage} - New Fis query message.
     */
    protected createMessage(messageParameter: MessageParameter): FisQueryMessage {
        try {
            this.queryMessage = {} as FisQueryMessage;
            return this.queryMessage;
        }
        catch (e) {
            throw e;
        }
    }
}

/**
 * Create new App Fis query message.
 *
 * @function createFisQueryMessage
 * @param messageParameter {MessageParameter} - Message parameters. 
 * @return {Message} - App Fis query message.
 */
export function createFisQueryMessage(messageParameter: MessageParameter): Message {
    try {
        return new AppFisQueryMessage().create(messageParameter);
    }
    catch (e) {
        throw e;
    }
}
