/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    AppMessageType, GenericHeader, GenericHeaderOf, AppMessageHeaderOptions, 
    FisQueryMessageParameter as MessageParameter, FisQueryMessageHeader
} from './fisrequestmessagetype';
import { AppMessageHeaderKind } from '../message/common/appmessageheaderkind';

/**
 * Fis query message header.
 *
 * @class AppFisQueryMessageHeader
 */
export class AppFisQueryMessageHeader extends AppMessageHeaderKind {
    /**
     * Create new Fis query message header.
     *
     * @class AppFisQueryMessageHeader
     * @param options {AppMessageHeaderOptions} - Message header options. 
     * @method constructor
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        // App message base header type is Fis request header.
        this.messageBaseHeaderType = AppMessageType.FisRequest;
        this.messageType = AppMessageType.FisQuery;
        this.permissibleMessageType = AppMessageType;
        if (!this.options) {this.options =  {}; }
        this.options.permissibleMessageType = this.permissibleMessageType;
    }

    /**
     * Create new Fis query header.
     *
     * @class AppFisQueryMessageHeader
     * @method createHeader
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisQueryMessageHeader} - New query header.
     */
    protected createHeader(messageParameter: MessageParameter): GenericHeader {
        try {
            let header: FisQueryMessageHeader = {} as FisQueryMessageHeader;
            header.query = (messageParameter && messageParameter.query);
            return header as GenericHeaderOf<FisQueryMessageHeader>;
        }
        catch (e) {
            throw e;
        }
    }
}
