/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    AppMessageType, MessageParameter, GenericHeader, GenericHeaderOf,
    AppMessageHeaderOptions, FisRequestMessageHeader,
} from './fisrequestmessagetype';
import { AppMessageHeaderKind } from '../message/common/appmessageheaderkind';
import { createFisMessageHeader } from './appfismessageheaderbase';

/**
 * Fis request message header.
 *
 * @class AppFisRequestMessageHeader
 */
export class AppFisRequestMessageHeader extends AppMessageHeaderKind {
    /**
     * Create new Fis request message header.
     *
     * @class AppFisRequestMessageHeader
     * @param options {AppMessageHeaderOptions} - Message header options. 
     * @method constructor
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        // App message base header type is Request.
        this.messageBaseHeaderType = AppMessageType.Request;
        this.messageType = AppMessageType.FisRequest;
        this.permissibleMessageType = AppMessageType;
        if (!this.options) {this.options =  {}; }
        this.options.permissibleMessageType = this.permissibleMessageType;
    }

    /**
     * Create new Fis request header.
     *
     * @class AppFisRequestMessageHeader
     * @method createHeader
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisRequestMessageHeader} - New request header.
     */
    protected createHeader(messageParameter: MessageParameter): GenericHeader {
        try {
            let header: FisRequestMessageHeader = {} as FisRequestMessageHeader;
            header.fisMessageHeader = createFisMessageHeader(messageParameter);  // Create new App Fis message header base.
            return header as GenericHeaderOf<FisRequestMessageHeader>;
        }
        catch (e) {
            throw e;
        }
    }
}
