/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    Message, AppMessageType, MessageParameter,  AppMessageHeaderOptions, 
    FisResponseMessage
} from './fisresponsemessagetype';
import { AppMessageKind } from '../message/common/appmessagekind';

/**
 * App Fis response message.
 *
 * @class AppFisResponseMessage
 */
export class AppFisResponseMessage extends AppMessageKind {
    /**
     * Create new app Fis response message.
     *
     * @class AppFisResponseMessage
     * @method constructor
     * @param options {AppMessageHeaderOptions} - Message options. 
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        this.messageType = AppMessageType.FisResponse;
    }

    /**
     * App Fis response Message.
     * 
     * @class AppFisResponseMessage
     * @property responseMessage
     * @type {FisResponseMessage} - App Fis response Message.
     */
    protected responseMessage: FisResponseMessage;

    /**
     * Create new Fis response message.
     *
     * @class AppFisResponseMessage
     * @method createMessage
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisResponseMessage} - New Fis response message.
     */
    protected createMessage(messageParameter: MessageParameter): FisResponseMessage {
        try {
            this.responseMessage = {} as FisResponseMessage;
            return this.responseMessage;
        }
        catch (e) {
            throw e;
        }
    }
}

/**
 * Create new App Fis response message.
 *
 * @function createFisResponseMessage
 * @param messageParameter {MessageParameter} - Message parameters. 
 * @return {Message} - App Fis response message.
 */
export function createFisResponseMessage(messageParameter: MessageParameter): Message {
    try {
        return new AppFisResponseMessage().create(messageParameter);
    }
    catch (e) {
        throw e;
    }
}
