/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    AppMessageType, MessageParameter, GenericHeader, GenericHeaderOf,
    AppResponseMessageHeaderOptions as AppMessageHeaderOptions, 
    FisResponseMessageHeader, RespondToMessageType
} from './fisresponsemessagetype'; 
import { AppMessageHeaderKind } from '../message/common/appmessageheaderkind';
import { createFisMessageHeader } from './appfismessageheaderbase';

/**
 * Fis response message header.
 *
 * @class AppFisResponseMessageHeader
 */
export class AppFisResponseMessageHeader extends AppMessageHeaderKind {
    /**
     * Create new Fis response message header.
     *
     * @class AppFisResponseMessageHeader
     * @param options {AppMessageHeaderOptions} - Message header options. 
     * @method constructor
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        // App message base header type is Response Header.
        this.messageBaseHeaderType = AppMessageType.Response;
        this.messageType = AppMessageType.FisResponse;
        this.permissibleMessageType = AppMessageType;
        if (!this.options) {this.options =  {}; }
        this.options.permissibleMessageType = this.permissibleMessageType;

        options = this.options;
        options.permissibleRespondToMessageType = RespondToMessageType; 
    }

    /**
     * Create new Fis response header.
     *
     * @class AppFisResponseMessageHeader
     * @method createHeader
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {FisResponseMessageHeader} - New response header.
     */
    protected createHeader(messageParameter: MessageParameter): GenericHeader {
        try {
            let header: FisResponseMessageHeader = {} as FisResponseMessageHeader;
            header.fisMessageHeader = createFisMessageHeader(messageParameter);  // Create new App Fis message header base.
            return header as GenericHeaderOf<FisResponseMessageHeader>;
        }
        catch (e) {
            throw e;
        }
    }

}
