/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    MessageHeader, FisMessageHeader, AppMessageType as BaseMessageType,
    FisMessageType, Command as CommandBase, FisCommand, Query as QueryBase,
    FisQuery, FisCommandMessage, FisQueryMessage
} from '../types/appmessagetype';

export * from '../types/appmessagetype';
export * from './fismessageparameter';

/**
 * Fis message header base.
 * Information required for message integration with FIS back office. 
 *
 */
export type FisMessageHeaderBase = MessageHeader & {
    fisMessageHeader: FisMessageHeader;
};

/**
 * Fis message base.
 * Information required for message integration with FIS back office. 
 *
 */
// fisMessage.serviceId = message.header.fisMessageHeader.serviceId;
// fisMessage.serviceInstanceId = message.header.fisMessageHeader.instanceId;

export interface FisMessageBase {
    header: FisMessageHeaderBase;
    data: unknown;
}

// Move to fisappmessageschema.json, Define in to fisappmessageschema as AppMessageType
/**
 * Fis Message types
 * 
 * @enum FisMessageType
 */
// export enum FisMessageType {
//     FisBase = "FisBase",
//     FisCommand = "FisCommand",
//     FisQuery = "FisQuery",
//     FisRequest = "FisRequest",
//     FisResponse = "FisResponse"
// };

export const AppMessageType = { ...BaseMessageType, ...FisMessageType }; // Merge message types

/**
 * JSON schema does not support inheritance.
 * Enum FisCommand need to extended from Enum Command.
 * Use Typescript Declaration Merging to merge Enum Command  into 
 * Enum FisCommand
 * Merger 
 */
export const Command = { ...CommandBase, ...FisCommand };

/**
 * JSON schema does not support inheritance.
 * Enum FisQUery need to extended from Enum Query.
 * Use Typescript Declaration Merging to merge Enum Query  into 
 * Enum FisQuery
 * Merger 
 */
export const Query = { ...QueryBase, ...FisQuery };

/**
 * FIS request type 
 * 
 * @enum FisRequestType
 */
export enum FisRequestType {
    /**
     * Command - Request for command service.
     */
    Command = "Command",

    /**
     * Query - Request for query service.
     */
    Query = "Query",

    /**
     * Response - Request for Response service.
     */
    Response = "Response"


};