/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import { RequestMessageParameter } from '../message/request/requestmessageparameter';
import { FisMessageParameter, FisCommandEnum, FisQueryEnum } from './fismessagetype';

/**
 * Fis request message parameter.
 * Consists of:
 *  Request message parameter.
 *  Fis message parameter.
 * 
 * @interface FisRequestMessageParameter 
 */
export interface FisRequestMessageParameter extends RequestMessageParameter, FisMessageParameter {
}

/**
 * Fis command message parameter.
 * Consists of:
 *  Fis request message parameter.
 *  Fis Command.
 * 
 * @interface FisCommandMessageParameter 
 */
export interface FisCommandMessageParameter extends FisRequestMessageParameter {
    /**
     * Fis command to perform.
     * 
     * @interface FisCommandMessageParameter
     * @property command
     * @type {FisCommand}
     */
    command: FisCommandEnum;
}

/**
 * Fis query message parameter.
 * Consists of:
 *  Fis request message parameter.
 *  Fis Query.
 * 
 * @interface FisQueryMessageParameter 
 */
export interface FisQueryMessageParameter extends FisRequestMessageParameter {
    /**
     * Fis query to perform.
     * 
     * @interface FisQueryMessageParameter
     * @property query
     * @type {FisQuery}
     */
    query: FisQueryEnum;
}

