/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    Message, AppMessageType, SubscriptionMessage, AppMessageHeaderOptions,
    SubscriptionMessageParameter as MessageParameter
} from './apprequestmessagetype';
import { AppMessageKind } from '../common/appmessagekind';

/**
 * App subscription message.
 *
 * @class AppSubscriptionMessage
 */
export class AppSubscriptionMessage extends AppMessageKind {
    /**
     * Create new app subscription message.
     *
     * @class AppSubscriptionMessage
     * @method constructor
     * @param options {AppMessageHeaderOptions} - Message options. 
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        this.messageType = AppMessageType.Subscription;
    }

    /**
     * App subscription Message.
     * 
     * @class AppSubscriptionMessage
     * @property subscriptionMessage
     * @type {SubscriptionMessage} - App subscription Message.
     */
    protected subscriptionMessage: SubscriptionMessage;

    /**
     * Create new subscription message.
     *
     * @class AppSubscriptionMessage
     * @method createMessage
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {SubscriptionMessage} - New subscription message.
     */
    protected createMessage(messageParameter: MessageParameter): SubscriptionMessage {
        try {
            this.subscriptionMessage = {} as SubscriptionMessage;
            return this.subscriptionMessage;
        }
        catch (e) {
            throw e;
        }
    }

}

/**
 * Create new App subscription message.
 *
 * @function createSubscriptionMessage
 * @param messageParameter {MessageParameter} - Message parameters. 
 * @return {Message} - App subscription message.
 */
export function createSubscriptionMessage(messageParameter: MessageParameter): Message {
    try {
        return new AppSubscriptionMessage().create(messageParameter);
    }
    catch (e) {
        throw e;
    }
}

