/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    AppMessageType, GenericHeader, GenericHeaderOf, AppMessageHeaderOptions,
    ResponseExceptionMessageParameter as MessageParameter,
    ResponseExceptionMessageHeader, ResponseException
} from './appresponsemessagetype';
import { AppMessageHeaderKind } from '../common/appmessageheaderkind';

/**
 * App response exception message header.
 *
 * @class AppResponseExceptionMessageHeader
 */
export class AppResponseExceptionMessageHeader extends AppMessageHeaderKind {
    /**
     * Create new app response exception message header.
     *
     * @class AppResponseExceptionMessageHeader
     * @param options {AppMessageHeaderOptions} - Message header options. 
     * @method constructor
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        // App message base header type is Response.
        this.messageBaseHeaderType = AppMessageType.Response;
        this.messageType = AppMessageType.ResponseException;
    }

    /**
     * Create new response exception header.
     *
     * @class AppResponseExceptionMessageHeader
     * @method createHeader
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {ResponseExceptionMessageHeader} - New response exception header.
     */
    protected createHeader(messageParameter: MessageParameter): GenericHeader {
        try {
            let header: ResponseExceptionMessageHeader = {} as ResponseExceptionMessageHeader;
            header.responseException = (messageParameter &&
                messageParameter.responseException) || ResponseException.InvalidRequest;
            return header as GenericHeaderOf<ResponseExceptionMessageHeader>;
        }
        catch (e) {
            throw e;
        }
    }
}
