/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import {
    Message, AppMessageType, MessageParameter, AppMessageHeaderOptions,
    ResponseSubscriptionMessage
} from './appresponsemessagetype';
import { AppMessageKind } from '../common/appmessagekind';

/**
 * App response subscription message.
 *
 * @class AppResponseSubscriptionMessage
 */
export class AppResponseSubscriptionMessage extends AppMessageKind {
    /**
     * Create new app response subscription message.
     *
     * @class AppResponseSubscriptionMessage
     * @method constructor
     * @param options {AppMessageHeaderOptions} - Message options. 
     */
    constructor(options?: AppMessageHeaderOptions) {
        super(options);
        this.messageType = AppMessageType.ResponseSubscription;
    }

    /**
     * App response subscription Message.
     * 
     * @class AppResponseSubscriptionMessage
     * @property responseMessage
     * @type {ResponseSubscriptionMessage}
     */
    protected responseSubscriptionMessage: ResponseSubscriptionMessage;

    /**
     * Create new response subscription message.
     *
     * @class AppResponseSubscriptionMessage
     * @method createMessage
     * @param messageParameter {MessageParameter} - Message parameters. 
     * @return {ResponseSubscriptionMessage} - New response subscription message.
     */
    protected createMessage(messageParameter: MessageParameter): ResponseSubscriptionMessage {
        try {
            this.responseSubscriptionMessage = {} as ResponseSubscriptionMessage;
            return this.responseSubscriptionMessage;
        }
        catch (e) {
            throw e;
        }
    }

}

/**
 * Create new App response subscription message.
 *
 * @function createResponseSubscriptionMessage
 * @param messageParameter {MessageParameter} - Message parameters. 
 * @return {Message} - App response subscription message.
 */
export function createResponseSubscriptionMessage(messageParameter: MessageParameter): Message {
    try {
        return new AppResponseSubscriptionMessage().create(messageParameter);
    }
    catch (e) {
        throw e;
    }
}