/*!
 * fisappmessagejs V1.0.0
 * Copyright(c) 2020 Software Optima Sdn. Bhd.
 */
import { FisCommandMessage, FisRequestType, AppMessageType } from '../../fis/fismessagetype';
import { AppMessageTransformerKind } from '../appmessagetransformerkind';
import { FisMessage } from './fismessage';

/**
 * App Fis command message transformer.
 * Transform to FisMessage
 *
 * @class  AppFisCommandMessageTransformer
 */
export class AppFisCommandMessageTransformer extends AppMessageTransformerKind {
    /**
     * Create new app Fis command message transformer.
     *
     * @class  AppFisCommandMessageTransformer
     * @method constructor
     */
    constructor() {
        super();
        this.baseTransformerType = [AppMessageType.FisBase];
        this.permissibleMessageType = AppMessageType.FisCommand;
    }

    /**
    * Transform app Fis command message extensions.
    *
    * @class AppFisCommandMessageTransformer
    * @method transformExtensions
    * @param message  {FisCommandMessage} - Fis command message. 
    * @return {object} - Output object..
    */
    protected transformExtensions(message: FisCommandMessage): object {
        let fisMessage: FisMessage = {} as FisMessage;
        fisMessage.operation = message.header.command;
        fisMessage.requestType = FisRequestType.Command;    // Request for command service.
        return fisMessage;
    }
}

/**
 *  Transform app Fis command message.
 *
 * @function transform
 * @param message  {FisCommandMessage} - Fis command Message. 
 * @return {FisMessage} - Fis message.
 */
export function transform(message: FisCommandMessage): FisMessage {
    try {
        return new AppFisCommandMessageTransformer().transform(message) as FisMessage;
    }
    catch (e) {
        throw e;
    }
}
